"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createStoreModule = createStoreModule;

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var CHANGE_STATE = 'CHANGE_STATE';
var CHANGE_ITEMS = 'CHANGE_ITEMS';
var CHANGE_ITEM = 'CHANGE_ITEM';

function createStoreModule(service, dataModel, log) {
  var _mutations;

  return {
    namespaced: true,
    state: function state() {
      return {
        loading: false,
        creating: false,
        updating: false,
        removing: false,
        next: null,
        item: _lodash.default.cloneDeep(dataModel),
        items: []
      };
    },
    mutations: (_mutations = {}, _defineProperty(_mutations, CHANGE_STATE, function (state, states) {
      _lodash.default.keys(states).forEach(function (s) {
        return state[s] = states[s];
      });
    }), _defineProperty(_mutations, CHANGE_ITEM, function (state, item) {
      state.item = _lodash.default.defaultsDeep({}, item, dataModel);
    }), _defineProperty(_mutations, CHANGE_ITEMS, function (state) {
      var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          items = _ref.items,
          next = _ref.next;

      state.items = _lodash.default.compact(items);
      state.next = next || null;
    }), _mutations),
    actions: {
      get: function () {
        var _get = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee(_ref2, query) {
          var state, commit;
          return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  state = _ref2.state, commit = _ref2.commit;

                  if (!(_lodash.default.isString(query) && state.item.id === query)) {
                    _context.next = 3;
                    break;
                  }

                  return _context.abrupt("return");

                case 3:
                  commit(CHANGE_STATE, {
                    loading: true
                  });
                  _context.prev = 4;
                  _context.t0 = commit;
                  _context.t1 = CHANGE_ITEM;

                  if (!_lodash.default.isString(query)) {
                    _context.next = 13;
                    break;
                  }

                  _context.next = 10;
                  return service.get(query);

                case 10:
                  _context.t2 = _context.sent;
                  _context.next = 16;
                  break;

                case 13:
                  _context.next = 15;
                  return service.findOne(query);

                case 15:
                  _context.t2 = _context.sent;

                case 16:
                  _context.t3 = _context.t2;
                  (0, _context.t0)(_context.t1, _context.t3);
                  _context.next = 24;
                  break;

                case 20:
                  _context.prev = 20;
                  _context.t4 = _context["catch"](4);
                  log(_context.t4);
                  commit(CHANGE_ITEM);

                case 24:
                  commit(CHANGE_STATE, {
                    loading: false
                  });

                case 25:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, this, [[4, 20]]);
        }));

        return function get(_x, _x2) {
          return _get.apply(this, arguments);
        };
      }(),
      find: function () {
        var _find = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee2(_ref3, query) {
          var state, commit;
          return regeneratorRuntime.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  state = _ref3.state, commit = _ref3.commit;
                  commit(CHANGE_STATE, {
                    loading: true
                  });
                  _context2.prev = 2;
                  _context2.t0 = commit;
                  _context2.t1 = CHANGE_ITEMS;
                  _context2.next = 7;
                  return service.find(query);

                case 7:
                  _context2.t2 = _context2.sent;
                  (0, _context2.t0)(_context2.t1, _context2.t2);
                  _context2.next = 15;
                  break;

                case 11:
                  _context2.prev = 11;
                  _context2.t3 = _context2["catch"](2);
                  log(_context2.t3);
                  commit(CHANGE_ITEMS);

                case 15:
                  commit(CHANGE_STATE, {
                    loading: false
                  });

                case 16:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2, this, [[2, 11]]);
        }));

        return function find(_x3, _x4) {
          return _find.apply(this, arguments);
        };
      }(),
      create: function () {
        var _create = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee3(_ref4, data) {
          var state, commit;
          return regeneratorRuntime.wrap(function _callee3$(_context3) {
            while (1) {
              switch (_context3.prev = _context3.next) {
                case 0:
                  state = _ref4.state, commit = _ref4.commit;

                  if (!state.creating) {
                    _context3.next = 3;
                    break;
                  }

                  return _context3.abrupt("return");

                case 3:
                  commit(CHANGE_STATE, {
                    creating: true
                  });
                  _context3.prev = 4;
                  _context3.t0 = commit;
                  _context3.t1 = CHANGE_ITEMS;
                  _context3.t2 = state.items;
                  _context3.next = 10;
                  return service.create(_objectSpread({}, data));

                case 10:
                  _context3.t3 = _context3.sent;
                  _context3.t4 = _context3.t2.concat.call(_context3.t2, _context3.t3);
                  _context3.t5 = state.next;
                  _context3.t6 = {
                    items: _context3.t4,
                    next: _context3.t5
                  };
                  (0, _context3.t0)(_context3.t1, _context3.t6);
                  _context3.next = 20;
                  break;

                case 17:
                  _context3.prev = 17;
                  _context3.t7 = _context3["catch"](4);
                  log(_context3.t7);

                case 20:
                  commit(CHANGE_STATE, {
                    creating: false
                  });

                case 21:
                case "end":
                  return _context3.stop();
              }
            }
          }, _callee3, this, [[4, 17]]);
        }));

        return function create(_x5, _x6) {
          return _create.apply(this, arguments);
        };
      }(),
      update: function () {
        var _update = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee4(_ref6, _ref5) {
          var state, commit, _ref5$id, id, changes, updated;

          return regeneratorRuntime.wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  state = _ref6.state, commit = _ref6.commit;
                  _ref5$id = _ref5.id, id = _ref5$id === void 0 ? state.item.id : _ref5$id, changes = _objectWithoutProperties(_ref5, ["id"]);

                  if (!state.updating) {
                    _context4.next = 4;
                    break;
                  }

                  return _context4.abrupt("return");

                case 4:
                  commit(CHANGE_STATE, {
                    updating: true
                  });
                  _context4.prev = 5;
                  _context4.next = 8;
                  return service.update(id, changes);

                case 8:
                  updated = _context4.sent;
                  if (!updated) commit(CHANGE_STATE, {
                    updating: false
                  });
                  if (id === state.item.id) commit(CHANGE_ITEM, updated);
                  commit(CHANGE_ITEMS, {
                    items: state.items.map(function (i) {
                      return i.id !== id ? i : updated;
                    }),
                    next: state.next
                  });
                  _context4.next = 17;
                  break;

                case 14:
                  _context4.prev = 14;
                  _context4.t0 = _context4["catch"](5);
                  log(_context4.t0);

                case 17:
                  commit(CHANGE_STATE, {
                    updating: false
                  });

                case 18:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4, this, [[5, 14]]);
        }));

        return function update(_x7, _x8) {
          return _update.apply(this, arguments);
        };
      }(),
      remove: function () {
        var _remove = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee5(_ref7) {
          var state,
              commit,
              id,
              removed,
              _args5 = arguments;
          return regeneratorRuntime.wrap(function _callee5$(_context5) {
            while (1) {
              switch (_context5.prev = _context5.next) {
                case 0:
                  state = _ref7.state, commit = _ref7.commit;
                  id = _args5.length > 1 && _args5[1] !== undefined ? _args5[1] : state.item.id;

                  if (!state.removing) {
                    _context5.next = 4;
                    break;
                  }

                  return _context5.abrupt("return");

                case 4:
                  commit(CHANGE_STATE, {
                    removing: true
                  });
                  _context5.prev = 5;
                  _context5.next = 8;
                  return service.remove(id);

                case 8:
                  removed = _context5.sent;

                  if (removed) {
                    _context5.next = 11;
                    break;
                  }

                  return _context5.abrupt("return", commit(CHANGE_STATE, {
                    removing: false
                  }));

                case 11:
                  if (id === state.item.id) commit(CHANGE_ITEM);
                  commit(CHANGE_ITEMS, {
                    items: state.items.filter(function (i) {
                      return i.id !== id;
                    }),
                    next: state.next
                  });
                  _context5.next = 18;
                  break;

                case 15:
                  _context5.prev = 15;
                  _context5.t0 = _context5["catch"](5);
                  log(_context5.t0);

                case 18:
                  commit(CHANGE_STATE, {
                    removing: false
                  });

                case 19:
                case "end":
                  return _context5.stop();
              }
            }
          }, _callee5, this, [[5, 15]]);
        }));

        return function remove(_x9) {
          return _remove.apply(this, arguments);
        };
      }()
    }
  };
}